//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * コリジョンノードヘッダ
 * @author Junpee
 */

#ifndef COLLISION_NODE_H_
#define COLLISION_NODE_H_

#include <Collision/System/CollisionObject.h>
#include <Core/Primitive/Axis3.h>
#include <Core/Container/ArrayList.h>

namespace Lamp{

class CollisionLeaf;

//------------------------------------------------------------------------------
/**
 * コリジョンノード
 */
class CollisionNode : public CollisionObject{
friend class CollisionScene;
public:
	//--------------------------------------------------------------------------
	// スケール
	//--------------------------------------------------------------------------
	/**
	 * スケールの設定
	 * @param scale スケール
	 */
	virtual void setScale(const Vector3& scale){ axis_.setScale(scale); }

	/**
	 * スケールの取得
	 * @return スケール
	 */
	virtual const Vector3& getScale() const{ return axis_.getScale(); }

	/**
	 * スケールを使用しているか
	 * @return スケールを使用しているならtrue
	 */
	virtual bool isScaled() const{ return axis_.isScaled(); }

	/**
	 * グローバルでスケールを使用しているか
	 * @return グローバルでスケールを使用しているならtrue
	 */
	virtual bool isGlobalScaled() const{ return globalScaled_; }

	//--------------------------------------------------------------------------
	// 回転
	//--------------------------------------------------------------------------
	/**
	 * XYZ回転の設定
	 * @param rotation XYZ回転
	 */
	virtual void setRotationXYZ(const Vector3& rotation){
		axis_.setRotationXYZ(rotation);
	}

	/**
	 * XYZ回転の取得
	 * @return XYZ回転
	 */
	virtual const Vector3& getRotationXYZ(){ return axis_.getRotationXYZ(); }

	//--------------------------------------------------------------------------
	/**
	 * 四元数回転の設定
	 * @param rotation 四元数回転
	 */
	virtual void setRotationQuaternion(const Quaternion& rotation){
		axis_.setRotationQuaternion(rotation);
	}

	/**
	 * 四元数回転の取得
	 * @return 四元数回転
	 */
	virtual const Quaternion& getRotationQuaternion(){
		return axis_.getRotationQuaternion();
	}

	//--------------------------------------------------------------------------
	// 移動
	//--------------------------------------------------------------------------
	/**
	 * 移動の設定
	 * @param translation 移動
	 */
	virtual void setTranslation(const Vector3& translation){
		axis_.setTranslation(translation);
	}

	/**
	 * 移動の取得
	 * @return 移動
	 */
	virtual const Vector3& getTranslation() const{
		return axis_.getTranslation();
	}

	//--------------------------------------------------------------------------
	// 行列
	//--------------------------------------------------------------------------
	/**
	 * ワールド行列の取得
	 * @return ワールド行列
	 */
	const Matrix34& getWorldMatrix() const{ return worldMatrix_; }

	/**
	 * ローカル行列の取得
	 * @return ローカル行列
	 */
	const Matrix34& getLocalMatrix() const{ return axis_.getMatrix(); }

	//--------------------------------------------------------------------------
	// 子供インターフェース
	//--------------------------------------------------------------------------
	/**
	 * 子供の追加
	 * @param child 追加する子供
	 */
	virtual void addChild(CollisionObject* child){
		child->setParent(this);
		children_.add(child);
	}

	/**
	 * 子供の削除
	 * @param child 削除する子供
	 */
	virtual void removeChild(CollisionObject* child){
		child->removeParent(this);
		children_.removeByValue(child);
	}

	/**
	 * 子供数の取得
	 * @return 子供数
	 */
	virtual int getChildCount() const{ return children_.getCount(); }

	/**
	 * 子供の取得
	 * @param index インデックス
	 * @return 子供
	 */
	virtual CollisionObject* getChild(int index) const{
		Assert(index >= 0);
		Assert(index < getChildCount());
		return children_.get(index);
	}

	//--------------------------------------------------------------------------
	// コピー
	//--------------------------------------------------------------------------
	/**
	 * コピー
	 * @return コピーされたコリジョンオブジェクト
	 */
	virtual CollisionObject* copy() const{ return copyCollisionNode(); }

	/**
	 * コリジョンノードのコピー
	 * @return コピーされたコリジョンノード
	 */
	virtual CollisionNode* copyCollisionNode() const;

	//--------------------------------------------------------------------------
	/**
	 * 再帰的破棄
	 * @param collisionNode 破棄するコリジョンノード
	 * @return 破棄したオブジェクト数
	 */
	static int recursiveDestroy(CollisionNode* collisionNode);

	/**
	 * 子の再帰的破棄
	 * @param collisionNode 子を破棄するコリジョンノード
	 * @return 破棄したオブジェクト数
	 */
	static int recursiveDestroyChildren(CollisionNode* collisionNode);

	//--------------------------------------------------------------------------
	// RTTI
	//--------------------------------------------------------------------------
	/**
	 * コリジョンノードかどうか
	 * @return コリジョンノードならtrue
	 */
	virtual bool isCollisionNode() const{ return true; }

protected:
	//--------------------------------------------------------------------------
	// 生成、破棄
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 * @param name 名前
	 * @param scene シーン
	 */
	CollisionNode(const String& name, CollisionScene* scene);

	/**
	 * デストラクタ
	 */
	virtual ~CollisionNode();

	//--------------------------------------------------------------------------
	// 走査
	//--------------------------------------------------------------------------
	/**
	 * 走査
	 * @param parentMatrix 親行列
	 * @param parentEnabled 親が有効か
	 * @param parentScaled 親がスケールを使用しているか
	 * @param parentChanged 親に変更があったか
	 */
	virtual void traverseImplement(const Matrix34& parentMatrix,
		bool parentEnabled, bool parentScaled, bool parentChanged);

	//--------------------------------------------------------------------------
	// 親
	//--------------------------------------------------------------------------
	/**
	 * 親の設定
	 * @param parent 設定する親
	 */
	virtual void setParent(CollisionNode* parent){
		CollisionObject::setParent(parent);
		setGlobalEnabled(true);
	}

	/**
	 * 親の削除
	 * @param parent 削除する親
	 */
	virtual void removeParent(CollisionNode* parent){
		CollisionObject::removeParent(parent);
		setGlobalEnabled(true);
	}

	//--------------------------------------------------------------------------
	// 変更フラグ
	//--------------------------------------------------------------------------
	/**
	 * 変更フラグの取得
	 * @return 変更されていればtrue
	 */
	virtual bool isChanged() const{
		return (CollisionObject::isChanged() || axis_.isChanged());
	}

	//--------------------------------------------------------------------------
	// スケール
	//--------------------------------------------------------------------------
	/**
	 * グローバルでスケールを使用しているかの設定
	 * @param globalScaled グローバルでスケールを使用しているならtrue
	 */
	virtual void setGlobalScaled(bool globalScaled){
		globalScaled_ = globalScaled;
	}

private:
	//--------------------------------------------------------------------------
	// ワールド行列
	Matrix34 worldMatrix_;
	// 軸
	Axis3 axis_;
	// 子供配列
	ArrayList<CollisionObject*> children_;
	// グローバルでのスケールフラグ
	bool globalScaled_;

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of COLLISION_NODE_H_
//------------------------------------------------------------------------------
